function [] = convertProfilesToRfb()
%CONVERTPROFILESTORFB Defines the profile to rfb conversion tool.

global artoaGui;

%% Initialize required variables

windowTitle = 'ARTOA4 - Profile to RFB converter';


%% Initialize gui

artoaGui.figures.convertProfilesToRfb = figure( ...
    'Name', windowTitle, ...
    'NumberTitle', 'off' ...
);


artoaGui.convertProfilesToRfb = struct();

set( ...
    artoaGui.figures.convertProfilesToRfb, ...
    'CloseRequestFcn', ...
    @artoa.controller.file.profile2rfb.close, ...
    'Units', 'normalized' ...
);

%% Generate Controls

left = .05;
width = .9;

%% Source frame
artoaGui.convertProfilesToRfb.frameSource = uipanel( ...
    'Parent', artoaGui.figures.convertProfilesToRfb, ...
    'Title', 'Source folder (*.profiles)', ...
    'Units', 'normalized', ...
    'BackgroundColor', 'white', ...
    'Position', [left .65 width .3] ...
);

artoaGui.convertProfilesToRfb.buttonSelectFolder = uicontrol( ...
    'Parent', artoaGui.convertProfilesToRfb.frameSource, ...
    'String', 'Pick a folder', ...
    'Style', 'PushButton', ...
    'FontSize', 8, ...
    'Units', 'normalized', ...
    'Position', [.7 .5 .25 .3], ...
    'Callback', @selectSourceFolder ...
);

artoaGui.convertProfilesToRfb.textSourceFolder = uicontrol( ...
    'Parent', artoaGui.convertProfilesToRfb.frameSource, ...
    'Style', 'edit', ...
    'FontSize', 8, ...
    'Units', 'normalized', ...
    'Enable', 'off', ...
    'Position', [.05 .5 .6 .3] ...
);

artoaGui.convertProfilesToRfb.buttonSelectMetadata = uicontrol( ...
    'Parent', artoaGui.convertProfilesToRfb.frameSource, ...
    'String', 'Pick metadata file', ...
    'Style', 'PushButton', ...
    'FontSize', 8, ...
    'Units', 'normalized', ...
    'Position', [.7 .1 .25 .3], ...
    'Callback', @selectSourceMetadata ...
);

artoaGui.convertProfilesToRfb.textSourceMetadata = uicontrol( ...
    'Parent', artoaGui.convertProfilesToRfb.frameSource, ...
    'Style', 'edit', ...
    'FontSize', 8, ...
    'Units', 'normalized', ...
    'Enable', 'off', ...
    'Position', [.05 .1 .6 .3] ...
);

%% Target frame
artoaGui.convertProfilesToRfb.frameTarget = uipanel( ...
    'Parent', artoaGui.figures.convertProfilesToRfb, ...
    'Title', 'Target file (*.rfb)', ...
    'Units', 'normalized', ...
    'BackgroundColor', 'white', ...
    'Position', [left .4 width .2] ...
);

artoaGui.convertProfilesToRfb.buttonSelectTargetFile = uicontrol( ...
    'Parent', artoaGui.convertProfilesToRfb.frameTarget, ...
    'String', 'Select a file', ...
    'Style', 'PushButton', ...
    'FontSize', 8, ...
    'Units', 'normalized', ...
    'Position', [.7 .4 .25 .4], ...
    'Callback', @selectTargetFile ...
);

artoaGui.convertProfilesToRfb.textTargetFile = uicontrol( ...
    'Parent', artoaGui.convertProfilesToRfb.frameTarget, ...
    'Style', 'edit', ...
    'FontSize', 8, ...
    'Units', 'normalized', ...
    'Enable', 'off', ...
    'Position', [.05 .4 .6 .4] ...
);

%% Execute frame
artoaGui.convertProfilesToRfb.frameExecute = uipanel( ...
    'Parent', artoaGui.figures.convertProfilesToRfb, ...
    'Title', 'Convert', ...
    'Units', 'normalized', ...
    'BackgroundColor', 'white', ...
    'Position', [left .1 width .2] ...
);

artoaGui.convertProfilesToRfb.textStatus = uicontrol( ...
    'Parent', artoaGui.convertProfilesToRfb.frameExecute, ...
    'String', 'Please input source and target...', ...
    'Style', 'Text', ...
    'FontSize', 8, ...
    'Units', 'normalized', ...
    'Position', [.1 .4 .35 .4] ...
);

artoaGui.convertProfilesToRfb.buttonExecuteConversion = uicontrol( ...
    'Parent', artoaGui.convertProfilesToRfb.frameExecute, ...
    'String', 'Start conversion', ...
    'Style', 'PushButton', ...
    'FontSize', 8, ...
    'Units', 'normalized', ...
    'Enable', 'off', ...
    'Position', [.5 .4 .3 .4], ...
    'Callback', @convert ...
);

%% Helper functions

    function selectSourceFolder(~, ~)
        dirname = uigetdir(pwd, 'Select source folder');
        if (dirname == 0)
            return
        end
        artoaGui.convertProfilesToRfb.textSourceFolder.String = dirname;
        checkInputs();
    end

    function selectSourceMetadata(~, ~)
        [filename, path] = uigetfile('*.xml', 'Select metadata file');
        if (filename == 0)
            return
        end
        artoaGui.convertProfilesToRfb.textSourceMetadata.String = ...
            fullfile(path, filename);
        checkInputs();
    end

    function selectTargetFile(~, ~)
        [filename, path] = uiputfile('*.rfb', 'Select target file');
        if (filename == 0)
            return
        end
        artoaGui.convertProfilesToRfb.textTargetFile.String = ...
            fullfile(path, filename);
        checkInputs();
    end

    function checkInputs(~, ~)
        if all(strcmp({ ...
                artoaGui.convertProfilesToRfb.textSourceFolder.String, ...
                artoaGui.convertProfilesToRfb.textTargetFile.String, ...
                artoaGui.convertProfilesToRfb.textSourceMetadata.String ...
                }, '') == 0)
            artoaGui.convertProfilesToRfb.buttonExecuteConversion.Enable = 'on';
            setStatus('Ready for conversion!');
        else
            artoaGui.convertProfilesToRfb.buttonExecuteConversion.Enable = 'off';
            setStatus('Please input source and target...');
        end
    end

    function setStatus(pStatusMessage)
        artoaGui.convertProfilesToRfb.textStatus.String = pStatusMessage;
        drawnow;
    end

    function convert(~, ~)
        setStatus('Loading files...');
        metadata = artoaGui.convertProfilesToRfb.textSourceMetadata.String;
        inputFolder = artoaGui.convertProfilesToRfb.textSourceFolder.String;
        target = artoaGui.convertProfilesToRfb.textTargetFile.String;
        float = nemo.load.single_float(inputFolder, filesep);
        setStatus('Creating rfb structure...');
        [rfb, ~] = nemo.rfb.create(float, nemo.load.float_metadata(metadata));
        setStatus('Save to target...');
        if ~nemo.rfb.save(rfb, target, true)
            setStatus('Something failed during conversion!');
        end
        setStatus('...success!');
    end

end

