function [ nemo_struct, startup_messages ] = floats2nemostruct( p_config, p_metadata )
%SBD2OPTIMARE Converts and combines the float .sbd data into the nemo structure.
%   Uses nemo.sbd.decode_float function to decode the profiles and
%   nemo.convert.combined_float2nemostruct to convert the combined profiles
%   to the nemo structure.
%
%   Parameters:
%   p_config:       The nemo configuration.
%   p_metadata:     The metadata information about the floats to process.
%
%   Returns:
%   nemo_struct:    The resulting nemo struct.

%% Run through metadata and process every float
nemo_struct = cell(length(p_metadata), 1);
startup_messages = cell(length(p_metadata), 1);

if (isa(p_metadata, 'struct'))
    tmp = p_metadata;
    p_metadata = {};
    p_metadata{1} = tmp;
    clear tmp;
end

for i = 1:length(p_metadata)
    % copy all metadata to new struct
    current_float = p_metadata{i};
    imei_int = p_metadata{i}.GENERAL_INFO.IMEI;
    %serial_str = sprintf('%04d', p_metadata{i}.GENERAL_INFO.SERIAL);
    
    % collect all information from harddisk
    [decoded_sbd, sbd_messages_info] = nemo.sbd.decode_float(p_config, imei_int);
    current_float.DECODED_SBD = decoded_sbd;
    current_float.SBD_MESSAGES_INFO = sbd_messages_info;
    transmission_info = nemo.load.float_transmission_info(p_config, imei_int);
    current_float.TRANSMISSION_INFO = transmission_info;
    
    % convert startup messages
    startup_messages{i} = nemo.float.extract_startup_message(current_float);
    % convert this information to the nemo structure
    nemo_struct{i} = nemo.convert.combined_float2nemostruct(current_float);
    % add startup message
    if ~isempty(startup_messages{i}) ...
       && ~isempty(startup_messages{i}{end}) ...
       && ~isa(nemo_struct{i}, 'logical')
        nemo_struct{i} = cellfun(@add_startup_message, nemo_struct{i}, 'UniformOutput', false);
    end
    
end

    function [ p_float ] = add_startup_message(p_float)
        p_float.STARTUP_MESSAGE = startup_messages{i}{end};
    end

end

