function S = decode_nemo(S)

% Decode different NEMO message types
%
%

% Optimare Sensorsysteme, 22.10.2010 M.Busack


% Enable(1)/Disable(0) 3830 Optode sensor
funco2 = 1;

switch S.DEVICE
    
    case 'CTD Profiler/Logger'
        % Device type = CTD-Profiler/Logger
        switch S.DATATYPE

            case 'Profile Message'
                % Read profile header
                S.SERIAL = S.uD(4)*256 + S.uD(3);
                S.PROFILE_NUMBER = S.uD(2)*256 + S.uD(1);
                S.STATUS_UPCAST = nemo.optimare.stat2str(S.uD(5),'C');
                % Startdate bytes 6,7,8 not used now
                S.DS4_START = S.uD(9)*3600 + S.uD(10)*60 + S.uD(11);
                S.BATT_CPU = S.uD(12) / 10;
                S.BATT_PMP = S.uD(13) / 10;
                S.PRES_TUBE_DEPTH = ( S.uD(14) / 10 ) * 68.9475;
                % Enddate bytes 15,16,17 not used now
                S.DS4_STOP = S.uD(18)*3600 + S.uD(19)*60 + S.uD(20);
                S.BATT_CPU2 = S.uD(21) / 10;
                S.BATT_PMP2 = S.uD(22) / 10;
                S.PRES_TUBE_SURFACE = ( S.uD(23) / 10 ) * 69.9475;
                %S.DATA_RECORDS_COMPLETE = S.uD(25)*256 + S.uD(24);
                S.DATA_RECORDS1 = [0, 0, 0, S.uD(25)*256+S.uD(24), 0];
                S.DATA_RECORDS2 = S.DATA_RECORDS1;
                offset = 26;
                % Read sample data
                if S.DATA_RECORDS1(4) > 0
                    for k=1:S.DATA_RECORDS1(4)
                        S.DS4_PRES(k) = nemo.optimare.cnv.press(S.uD, offset, 2);
                        S.DS4_TEMP(k) = nemo.optimare.cnv.temp(S.uD, offset+2, 2);
                        S.DS4_SAL(k) = nemo.optimare.cnv.salt(S.uD, offset+4, 2);
                        offset = offset + 6;
                    end
                end

        end

        
    case 'NEMO Standard      '
        % Device type = NEMO float

        switch S.DATATYPE

            case 'Startup Test Message'
                S.SERIAL = S.uD(2)*256 + S.uD(1);
                dt = [S.uD(3)+2000,S.uD(4),S.uD(5),0,0,0];
                S.SOFTWARE = datestr(dt);
                dt = [S.uD(6)+2000,S.uD(7),S.uD(8),S.uD(9),S.uD(10),0];
                S.RTC = datestr(dt);
                S.PISTON_MAX = S.uD(12)*256 + S.uD(11);
                S.PISTON_MIN = S.uD(14)*256 + S.uD(13);
                S.BATT_CPU = S.uD(15) / 10;
                S.BATT_PMP = S.uD(16) / 10;
                S.BATT_EXT = S.uD(17) / 10;
                S.PRES_TUBE_SURFACE = (S.uD(18) / 10) * 68.94757;
                S.PRES_PROFILE = nemo.optimare.cnv.press(S.uD, 19, 2);
                S.PRES_PARKING = nemo.optimare.cnv.press(S.uD, 21, 2);
                S.TIME_TX = S.uD(24)*256 + S.uD(23);
                S.TIME_PARKSAMPLE = S.uD(26)*256 + S.uD(25);
                S.TIME_CYCLE = S.uD(28)*256 + S.uD(27);
                S.DESCENT_MODE = S.uD(29);
                S.RECOVERY_START = S.uD(32)*256 + S.uD(31);
                S.DESCENT_SPEED = (S.uD(34)*256 + S.uD(33))/1000;
                S.TEMP_ICEDETECT = (S.sD(36)*256 + S.uD(35))/1000;
                S.RECOVERY_TX = S.uD(37);
                S.RECOVERY_NOTX = S.uD(40)*256 + S.uD(39);
                dt = [S.uD(41)+2000,S.uD(42),S.uD(43),S.uD(44),S.uD(45),0];
                S.GPS_DATETIME = datestr(dt);
                S.GPS_LAT = (S.sD(48)*256 + S.uD(47)) / 100;
                S.GPS_LON = (S.sD(50)*256 + S.uD(49)) / 100;

            case 'Profile Message'
                % Read profile header (ab rev.224)
                S.SERIAL = S.uD(4)*256 + S.uD(3);
                S.PROFILE_NUMBER = S.uD(2)*256 + S.uD(1);
                S.STATUS_UPCAST = nemo.optimare.stat2str(S.uD(5),'P');
                % Byte 6 nicht benutzt
                S.PROFILE_OLDER = S.uD(8)*256 + S.uD(7);
                S.ERROR_MOTOR = S.uD(9);
                S.ERROR_CTD = S.uD(10);
                S.ERROR_OPTODE = S.uD(11);
                S.ERROR_RAFOS = S.uD(12);
                S.BATT_CPU = S.uD(13) / 10;
                S.BATT_PMP = S.uD(14) / 10;
                S.BATT_EXT = S.uD(15) / 10;
                S.CURR_MOTOR = S.uD(16) / 100;
                S.CURR_MOTOR_MEAN = S.uD(17) / 100;
                if isnan(S.uD(18))
                    S.CTD_MODE = NaN;
                    S.CTD_RES = NaN;
                else
                    S.CTD_MODE = bitget(S.uD(18),1);
                    S.CTD_RES = bitget(S.uD(18),2);
                end
                S.PRES_TUBE_SURFACE = S.uD(20)*256 + S.uD(19);
                S.TEMP_TUBE_SURFACE = (S.sD(22)*256 + S.uD(21))/10;
                S.PRES_TUBE_SURFACE2 = S.uD(24)*256 + S.uD(23);
                S.TEMP_TUBE_SURFACE2 = (S.sD(26)*256 + S.uD(25))/10;
                S.PRES_TUBE_DEPTH = S.uD(28)*256 + S.uD(27);
                S.TEMP_TUBE_DEPTH = (S.sD(30)*256 + S.uD(29))/10;
                S.TEMP_SHT = (S.sD(32)*256 + S.uD(31)) / 10;
                S.HUM_SHT = (S.uD(34)*256 + S.uD(33)) / 10;
                S.PRES_OFFSET = nemo.optimare.cnv.press(S.uD, 35, 2);
                S.PRES_SURFACE = nemo.optimare.cnv.press(S.uD, 37, 2); 
                S.PRES_PARKING = nemo.optimare.cnv.press(S.uD, 39, 2);
                S.PRES_DEPTH = nemo.optimare.cnv.press(S.uD, 41, 2);
                S.PRES_MAX = nemo.optimare.cnv.press(S.uD, 43, 2);
                S.RECOVERY_START = S.uD(46)*256 + S.uD(45);
                S.DATA_RECORDS1 = [S.uD(48)*256+S.uD(47), S.uD(50)*256+S.uD(49),...
                    S.uD(52)*256+S.uD(51), S.uD(54)*256+S.uD(53), S.uD(56)*256+S.uD(55)];
                S.PIST_SURFACE = S.uD(58)*256 + S.uD(57);
                S.PIST_PARKING = S.uD(60)*256 + S.uD(59);
                S.PIST_CALC = S.uD(62)*256 + S.uD(61);
                S.PIST_DEPTH = S.uD(64)*256 + S.uD(63);
                S.PIST_EOP = S.uD(66)*256 + S.uD(65);
                S.MCNT_DESCENT = S.uD(70)*16777216 + S.uD(69)*65536 + S.uD(68)*256 + S.uD(67);
                S.MCNT_PARKING = S.uD(74)*16777216 + S.uD(73)*65536 + S.uD(72)*256 + S.uD(71);
                S.MCNT_SAG = S.uD(78)*16777216 + S.uD(77)*65536 + S.uD(76)*256 + S.uD(75);
                S.MCNT_ASCENT = S.uD(82)*16777216 + S.uD(81)*65536 + S.uD(80)*256 + S.uD(79);
                S.MCNT_ASCENT_END = S.uD(86)*16777216 + S.uD(85)*65536 + S.uD(84)*256 + S.uD(83);
                S.MCNT_SURFACE = S.uD(90)*16777216 + S.uD(89)*65536 + S.uD(88)*256 + S.uD(87);
                S.DATA_RECORDS2 = [S.uD(92)*256+S.uD(91), S.uD(94)*256+S.uD(93),...
                    S.uD(96)*256+S.uD(95), S.uD(98)*256+S.uD(97), S.uD(100)*256+S.uD(99)];
                % Verify data record quantity
                for k = 1:5
                    if ( (S.DATA_RECORDS1(k) == 0) && (S.DATA_RECORDS2(k) > 0) )
                        q(k) = S.DATA_RECORDS2(k);
                    else
                        q(k) = S.DATA_RECORDS1(k);
                    end
                end
                % Set offset in array and resulution increment
                offset = 101;
                if S.CTD_RES == 0
                    ctda = 2;
                else
                    ctda = 4;
                end
                % Read profile data sets (ab Rev.224)
                % Dataset 1 (profile header extensions)
                if q(1) > 0
                    for k=1:q(1)
                        % This record header is always there
                        S.DS1_TYPE(k) = S.uD(offset);
                        S.DS1_FORMAT(k) = S.uD(offset+1);
                        offset = offset + 2;
                        switch(S.DS1_TYPE(k))
                            case 1                      % Surface detect data only
                                if S.DS1_FORMAT(k) == 1 % Default format
                                    S.ICEGPS_TIME = S.uD(offset);
                                    S.ICEIRDM_TIME = S.uD(offset+1);
                                    offset = offset + 2;
                                end
                                % Paste additional formats for type 1 here
                                
                            case 2                      % Mission configuration data
                                [S,offset] = nemo.optimare.cnv.phext_t2(S,offset,k);
                                % Paste additional formats for type 2 here
                                
                            case 3                      % Airpump control data
                                if S.DS1_FORMAT(k) == 1 % Default format
                                    S.AIRPUMP_RUNTIME = S.uD(offset+1)*256 + S.uD(offset);
                                    offset = offset + 2;
                                end
                                % Paste additional formats for type 3 here
                                
                            case 4                      % Ice detection temperature data
                                if S.DS1_FORMAT(k) == 1 % Default format
                                    S.ICE_TEMPMEDIAN = ( S.sD(offset+3)*16777216 + S.uD(offset+2)*65536 +...
                                        S.uD(offset+1)*256 + S.uD(offset) ) / 10000;
%                                     S.ICE_TEMPMEDIAN = ( S.uD(offset+3)*16777216 + S.uD(offset+2)*65536 +...
%                                         S.uD(offset+1)*256 + S.uD(offset) ) / 10000;
                                    offset = offset + 4;
                                end
                                % Paste additional formats for type 4 here
                                
                            % Paste additional types here
                            
                            otherwise
                                % Do nothing
                                
                        end
                    end         
                end
                % Dataset 2 (Parking data)
                if q(2) > 0
                    for k=1:q(2)
                        S.DS2_PRES(k) = nemo.optimare.cnv.press(S.uD, offset, ctda);
                        S.DS2_TEMP(k) = nemo.optimare.cnv.temp(S.uD, offset+ctda, ctda);
                        S.DS2_SAL(k) = nemo.optimare.cnv.salt(S.uD, offset+2*ctda, ctda);
                        offset = offset + 3*ctda;
                    end
                end
                % Dataset 3 (Rafos data)
                if q(3) > 0
                    for k=1:q(3)
                        S.DS3_STATUS(k) = S.uD(offset);
                        S.DS3_DATE(k) = S.uD(offset+1)*372 + S.uD(offset+2)*31 + S.uD(offset+3);
                        S.DS3_DATE_year(k) = S.uD(offset+1)*372 ;
                        S.DS3_DATE_month(k) =  S.uD(offset+2);
                        S.DS3_DATE_day(k) = S.uD(offset+3);

                        S.DS3_TIME(k) = S.uD(offset+4)*3600 + S.uD(offset+5)*60 + S.uD(offset+6);
                        for i=1:6
                            S.DS3_RAFOSAMP(k,i) = S.uD(offset+6+i);
                            S.DS3_RAFOSRANK(k,i) = (S.uD(offset+15+(i-1)*2)*256 + S.uD(offset+14+(i-1)*2))*0.3075;
                        end
                        % Evtl muss hier noch ein dummy byte eingefügt werden
                        % (ist im 160 0 00 00 502 schon drin)
                        % todo testen und in Firmware auch diesen datensatz
                        % evtl. mit high resulution !
                        S.DS3_PRES(k) = nemo.optimare.cnv.press(S.uD, offset+26, 2);
                        S.DS3_TEMP(k) = nemo.optimare.cnv.temp(S.uD, offset+28, 2);
                        S.DS3_SAL(k) = nemo.optimare.cnv.salt(S.uD, offset+30, 2);
                        offset = offset + 32;
                    end
                end
                % Dataset 4 (Upcast profile data)
                if q(4) > 0
                    for k=1:q(4)
                        S.DS4_TIME(k) = S.uD(offset)*3600 + S.uD(offset+1)*60 + S.uD(offset+2);
                        S.DS4_FORMAT(k) = S.uD(offset+3);
                        S.DS4_PRES(k) = nemo.optimare.cnv.press(S.uD, offset+4, ctda);
                        offset = offset + 4 + ctda;
                        if S.CTD_MODE == 0
                            % Discrete profile with full ctd data
                            S.DS4_TEMP(k) = nemo.optimare.cnv.temp(S.uD, offset, ctda);
                            S.DS4_SAL(k) = nemo.optimare.cnv.salt(S.uD, offset+ctda, ctda);
                            offset = offset + ctda*2;
                        else 
                            % Cont. profile or NaN
                            S.DS4_TEMP(k) = NaN;
                            S.DS4_SAL(k) = NaN;
                        end
                        if ((funco2 > 0) || (S.DS4_FORMAT(k) > 0) )
                            % Read additionaly sensor data
                            [S,offset] = nemo.optimare.read_sensors(S, k, offset); 
                        end
                    end
                end
                % Dataset 5 (Extended surface data)
                if q(5) > 0
                    for k=1:q(5)
                        dt = [S.uD(offset)+2000,S.uD(offset+1),S.uD(offset+2),S.uD(offset+3),S.uD(offset+4),S.uD(offset+5)];
                        %S.DS5_RTC(k) = datestr(dt);
                        S.DS5_RTC(k) = datenum(dt);
                        dt = [S.uD(offset+6)+2000,S.uD(offset+7),S.uD(offset+8),S.uD(offset+9),S.uD(offset+10),S.uD(offset+11)];
                        %S.DS5_GPS(k) = datestr(dt);
                        S.DS5_GPS(k) = datenum(dt);
                        S.DS5_LAT(k) = (S.sD(offset+15)*16777216 + S.uD(offset+14)*65536 + S.uD(offset+13)*256 + S.uD(offset+12)) / 10000000;
                        S.DS5_LON(k) = (S.sD(offset+19)*16777216 + S.uD(offset+18)*65536 + S.uD(offset+17)*256 + S.uD(offset+16)) / 10000000;
                        offset = offset + 20;
                    end
                end

            case 'MTM Report Message'



            case 'Recovery Message'



            case 'File transmission message'


            case 'Filtered CTD data Message'
                % Read continous profile data header (filtered)
                S.SERIAL = S.uD(2)*256 + S.uD(1);            
                S.PROFILE_NUMBER = S.uD(4)*256 + S.uD(3);
                S.DS4_START = S.uD(5)*3600 + S.uD(6)*60 + S.uD(7);
                S.DS4_STOP = S.uD(8)*3600 + S.uD(9)*60 + S.uD(10);
                S.DATA_RECORDS_COMPLETE = S.uD(12)*256 + S.uD(11);
                S.DATA_RECORDS1 = [0, 0, 0, S.uD(14)*256+S.uD(13), 0];
                S.DATA_RECORDS2 = S.DATA_RECORDS1;
                S.CTD_RES = bitget(S.uD(15),2);
                offset = 16;
                if S.CTD_RES == 0
                    ctda = 2;
                else
                    ctda = 4;
                end
                % Read sample data
                if S.DATA_RECORDS1(4) > 0
                    for k=1:S.DATA_RECORDS1(4)
                        S.DS4_PRES(k) = nemo.optimare.cnv.press(S.uD, offset, ctda);
                        S.DS4_TEMP(k) = nemo.optimare.cnv.temp(S.uD, offset+ctda, ctda);
                        S.DS4_SAL(k) = nemo.optimare.cnv.salt(S.uD, offset+2*ctda, ctda);
                        offset = offset + 3*ctda;
                    end
                end

        end

    otherwise
        % Unknown device type
        
end

% function end
end


    